/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.visualization.world;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JComponent;

import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tud.kom.p2psim.impl.topology.views.VisualizationTopologyView.VisualizationInjector;
import de.tud.kom.p2psim.impl.topology.views.visualization.ui.VisHelper;
import de.tudarmstadt.maki.simonstrator.api.common.graph.IEdge;
import de.tudarmstadt.maki.simonstrator.api.component.network.NetworkComponent.NetInterfaceName;
import de.tudarmstadt.maki.simonstrator.api.component.topology.TopologyID;

/**
 * Visualization of a Link
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 19.03.2012
 */
public class LinkComponentVis extends JComponent {
	
	private TopologyComponentVis src;
	
	public LinkComponentVis(TopologyComponentVis src) {
		this.src = src;
		setBounds(0, 0, VisualizationInjector.getWorldX(),
				VisualizationInjector.getWorldY());
		setOpaque(false);
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;

		Simulator.getScheduler().pause();
		TopologyID tId = src.getTopologyComp().getTopologyID(NetInterfaceName.WIFI, true);
		Set<IEdge> neighbors = new HashSet<IEdge>(src.getTopologyComp().getNeighbors(tId));
		for (IEdge mAddr : neighbors) {
			TopologyComponentVis dst = VisualizationInjector
					.getTopologyComponentVis(mAddr.toId());
			VisHelper.drawArrow(g2,
					VisualizationInjector.scaleValue(src.getPosition().getX()),
					VisualizationInjector.scaleValue(src.getPosition().getY()),
					VisualizationInjector.scaleValue(dst.getPosition().getX()),
					VisualizationInjector.scaleValue(dst.getPosition().getY()),
					2);
		}
		Simulator.getScheduler().unpause();
	}
	
}