/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.visualization.world;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;

import de.tud.kom.p2psim.api.topology.TopologyComponent;
import de.tud.kom.p2psim.impl.topology.PositionVector;
import de.tud.kom.p2psim.impl.topology.views.VisualizationTopologyView;
import de.tud.kom.p2psim.impl.topology.views.VisualizationTopologyView.VisualizationInjector;

/**
 * Visualization of a component
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, 19.03.2012
 */
public class TopologyComponentVis extends JComponent {

	/**
	 * 
	 */
	protected final VisualizationTopologyView visualizationTopologyView;

	protected final int COMPONENT_PADDING = 15;

	protected final int COMPONENT_RADIUS = 3;

	protected final Color COLOR_OFFLINE = new Color(100, 100, 100, 50);

	protected Color color_online = new Color(255, 0, 255, 255);

	private TopologyComponent tComp;

	private PositionVector posVec;
	
	protected final long hostid;

	private LinkComponentVis lcVis = null;

	protected boolean active = false;

	protected boolean clicked = false;

	public TopologyComponentVis(VisualizationTopologyView visualizationTopologyView, TopologyComponent comp) {
		this.visualizationTopologyView = visualizationTopologyView;
		tComp = comp;
		posVec = comp.getRealPosition();
		hostid = comp.getHost().getId().value();
		
		String peerColor = comp.getHost().getProperties().getProperty("peerColor");
		if (peerColor != null) {
			int cInt = Integer.parseInt(peerColor, 16);
			color_online = new Color(cInt);
		}
		
		Point position = comp.getRealPosition().asPoint();
		setBounds(
				VisualizationInjector.scaleValue(position.getX())
						- COMPONENT_PADDING,
				VisualizationInjector.scaleValue(position.getY())
						- COMPONENT_PADDING,
				2 * COMPONENT_PADDING, 2 * COMPONENT_PADDING);
		setOpaque(false);
		this.addMouseListener(new MouseAdapter() {

			/**
			 * Stores the mouse position, if the mouse button is pressed
			 */
			@Override
			public void mousePressed(MouseEvent e) {
				clicked = !clicked;
				TopologyComponentVis.this.visualizationTopologyView
						.notifyInteractionListenersOnClick(hostid, clicked);
				VisualizationInjector.getWorldPanel().invalidate();
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				if (!active)
					toggleActive();
			}

			@Override
			public void mouseExited(MouseEvent e) {
				if (active && !clicked) {
					toggleActive();
				}
			}

		});
	}

	/**
	 * Access to the visualized {@link TopologyComponent}
	 * 
	 * @return
	 */
	public TopologyComponent getTopologyComp() {
		return tComp;
	}

	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		setLocation(
				VisualizationInjector.scaleValue(posVec.getX())
						- COMPONENT_PADDING,
				VisualizationInjector.scaleValue(posVec.getY())
						- COMPONENT_PADDING);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		if (clicked) {
			g2.setColor(Color.MAGENTA);
		} else {
			g2.setColor(Color.DARK_GRAY);
		}

		if (tComp.getHost().getNetworkComponent() != null && !tComp.getHost().getNetworkComponent().getNetworkInterfaces()
				.iterator().next().isUp()) {
			g2.setColor(COLOR_OFFLINE);
		}

		g2.fillOval(COMPONENT_PADDING - COMPONENT_RADIUS, COMPONENT_PADDING
				- COMPONENT_RADIUS, COMPONENT_RADIUS * 2,
				COMPONENT_RADIUS * 2);


		g2.setColor(Color.BLACK);
		g2.setFont(VisualizationTopologyView.FONT_TINY);
		g2.drawString(String.valueOf(hostid), COMPONENT_PADDING,
				COMPONENT_PADDING - COMPONENT_RADIUS);
		g2.setColor(Color.BLACK);
	}

	protected void toggleActive() {
		active = !active;
		// if (active) {
		// lcVis = new LinkComponentVis(TopologyComponentVis.this);
		// VisualizationInjector.getWorldPanel().add(lcVis);
		// } else if (lcVis != null) {
		// VisualizationInjector.getWorldPanel().remove(lcVis);
		// }
		// VisualizationInjector.getWorldPanel().invalidate();
	}

	protected PositionVector getPosition() {
		return posVec;
	}

}